//
//  AMVersionNumber.m
//  TickerUpdate
//
//  Created by Andreas on 21.09.04.
//  Copyright 2004 Andreas Mayer. All rights reserved.
//

#import "AMVersionNumber.h"
#import "NSWorkspace_AMSystemVersion.h"


@interface AMVersionNumber (Private)
- (void)setMajorVersion:(unsigned)newMajorVersion;
- (void)setMinorVersion:(unsigned)newMinorVersion;
- (void)setRevision:(unsigned)newRevision;
- (void)setBuild:(NSString *)newBuild;
- (void)setBuildNumber:(unsigned)newBuildNumber;
- (NSComparisonResult)compareBuild:(NSString *)build1 toBuild:(NSString *)build2;
- (BOOL)isAlphaString:(NSString *)string;
- (BOOL)isBetaString:(NSString *)string;
@end

@implementation AMVersionNumber

+ (AMVersionNumber *)systemVersion
{
	unsigned ma, mi, rev;
	[NSWorkspace getSystemVersionMajor:&ma minor:&mi bugFix:&rev];
	return [[[AMVersionNumber alloc] initWithMajor:ma minor:mi revision:rev build:nil buildNumber:0] autorelease];
}

+ (AMVersionNumber *)versionNumberWithString:(NSString *)versionString
{
	return [[[AMVersionNumber alloc] initWithString:versionString] autorelease];
}

- (id)initWithMajor:(unsigned)ma minor:(unsigned)mi revision:(unsigned)rev build:(NSString *)bu buildNumber:(unsigned)num
{
	if ((self = [super init])) {
		[self setMajorVersion:ma];
		[self setMinorVersion:mi];
		[self setRevision:rev];
		[self setBuild:bu];
		[self setBuildNumber:num];
	}
	return self;
}

- (id)initWithString:(NSString *)versionString
{
	if (self = [super init]) {
		NSCharacterSet *romanDecimalDigitsCharacterSet = [NSCharacterSet characterSetWithCharactersInString:@"0123456789"];
		NSCharacterSet *dotCharacterSet = [NSCharacterSet characterSetWithCharactersInString:@"."];
		NSScanner *scanner = [NSScanner scannerWithString:versionString];
		NSString *part = [NSString string];
		int integerValue;
		// up to first dot
		if ([scanner scanInt:&integerValue]) {
			[self setMajorVersion:integerValue];
			[scanner setCharactersToBeSkipped:dotCharacterSet];
			if ([scanner scanInt:&integerValue]) {
				[self setMinorVersion:integerValue];
				if ([scanner scanInt:&integerValue]) {
					[self setRevision:integerValue];
				}
			}
		}
		if (![scanner isAtEnd]) {
			// take rest as build
			if ([scanner scanUpToCharactersFromSet:romanDecimalDigitsCharacterSet intoString:&part]) {
				[self setBuild:part];
				if ([scanner scanInt:&integerValue]) {
					[self setBuildNumber:integerValue];
				}
			}
		}
	}
	return self;
}


- (NSComparisonResult)compare:(id)otherVersion
{
	NSComparisonResult result = NSOrderedSame;
	if ([otherVersion isKindOfClass:[AMVersionNumber class]]) {
		result = [self compareToVersionNumber:otherVersion];
	} else if ([otherVersion isKindOfClass:[NSString class]]) {
		result = [self compareToVersionNumber:[AMVersionNumber versionNumberWithString:otherVersion]];
	} else {
		// exception
		[NSException raise:NSBadComparisonException format:@"%@ is not kind of class AMVersionNumber or NSString", otherVersion];
	}
	return result;
}

- (NSComparisonResult)compareToVersionNumber:(AMVersionNumber *)otherVersion
{
	NSComparisonResult result = NSOrderedDescending;
	if ([self majorVersion] == [otherVersion majorVersion]) {
		if ([self minorVersion] == [otherVersion minorVersion]) {
			if ([self revision] == [otherVersion revision]) {
				NSComparisonResult buildCompare = ([self compareBuild:[self build] toBuild:[otherVersion build]]);
				if (buildCompare == NSOrderedSame) {
					if ([self buildNumber] == [otherVersion buildNumber]) {
						result = NSOrderedSame;
					} else if ([self buildNumber] < [otherVersion buildNumber]) {
						result = NSOrderedAscending;
					}
				} else {
					result = buildCompare;
				}
			} else if ([self revision] < [otherVersion revision]) {
				result = NSOrderedAscending;
			}
		} else if ([self minorVersion] < [otherVersion minorVersion]) {
			result = NSOrderedAscending;
		}
	} else if ([self majorVersion] < [otherVersion majorVersion]) {
		result = NSOrderedAscending;
	}
	return result;
}

- (BOOL)isEqualTo:(id)otherVersion
{
	BOOL result = NO;
	if ([otherVersion isKindOfClass:[AMVersionNumber class]]) {
		result = ([self compareToVersionNumber:otherVersion] == NSOrderedSame);
	} else if ([otherVersion isKindOfClass:[NSString class]]) {
		result = ([self compareToVersionNumber:[AMVersionNumber versionNumberWithString:otherVersion]] == NSOrderedSame);
	}
	return result;
}

- (NSComparisonResult)compareBuild:(NSString *)build1 toBuild:(NSString *)build2
{
	NSComparisonResult result;
	if ([self isAlphaString:build1]) {
		if ([self isAlphaString:build2]) {
			result = NSOrderedSame;
		} else {
			result = NSOrderedAscending;
		}
	} else if ([self isBetaString:build1]) {
		if ([self isAlphaString:build2]) {
			result = NSOrderedDescending;
		} else if ([self isBetaString:build2]) {
			result = NSOrderedSame;
		} else {
			result = NSOrderedAscending;
		}
	} else {
		if (([self isAlphaString:build2]) || ([self isBetaString:build2])) {
			result = NSOrderedDescending;
		} else {
			result = [build1 compare:build2];
		}
	}
	return result;
}

- (BOOL)isAlphaString:(NSString *)string
{
	BOOL result = NO;
	if ([[string lowercaseString] isEqualToString:@"alpha"]) {
		result = YES;
	} else if ([string isEqualToString:[NSString stringWithUTF8String:"α"]]) {
		result = YES;
	}
	return result;
}

- (BOOL)isBetaString:(NSString *)string
{
	BOOL result = NO;
	if ([[string lowercaseString] isEqualToString:@"beta"]) {
		result = YES;
	} else if ([string isEqualToString:[NSString stringWithUTF8String:"β"]]) {
		result = YES;
	} else if ([string isEqualToString:[NSString stringWithUTF8String:"ß"]]) {
		result = YES;
	}
	return result;
}


- (NSString *)stringValue
{
	NSMutableString *result = nil;
	if ([self revision] > 0) {
		result = [[NSMutableString alloc] initWithFormat:@"%i.%i.%i", [self majorVersion], [self minorVersion], [self revision]];
	} else {
		result = [[NSMutableString alloc] initWithFormat:@"%i.%i", [self majorVersion], [self minorVersion]];
	}
	if ([self build]) {
		[result appendString:[self build]];
		if ([self buildNumber] > 0) {
			[result appendString:[NSString stringWithFormat:@"%i", [self buildNumber]]];
		}
	} else if ([self buildNumber] > 0) {
		[result appendString:[NSString stringWithFormat:@".%i", [self buildNumber]]];
	}
	return [result autorelease];
}

- (NSString *)description
{
	return [self stringValue];
}

- (unsigned)majorVersion
{
	return major;
}

- (void)setMajorVersion:(unsigned)newMajorVersion
{
	major = newMajorVersion;
}

- (unsigned)minorVersion
{
	return minor;
}

- (void)setMinorVersion:(unsigned)newMinorVersion
{
	minor = newMinorVersion;
}

- (unsigned)revision
{
	return revision;
}

- (void)setRevision:(unsigned)newRevision;
{
	revision = newRevision;
}

- (NSString *)build
{
	return build; 
}

- (void)setBuild:(NSString *)newBuild
{
	if (build != newBuild) {
		[newBuild retain];
		[build release];
		build = newBuild;
	}
}

- (unsigned)buildNumber
{
	return buildNumber;
}

- (void)setBuildNumber:(unsigned)newBuildNumber
{
	buildNumber = newBuildNumber;
}


@end
