//
//  NSImageAMAdditions.m
//  TimeDiscSaver
//
//  Created by Andreas on Sat Jan 18 2003.
//  Copyright (c) 2003 Andreas Mayer. All rights reserved.
//
//	2004-07-11	Andreas Mayer
//	- works with images without alpha channel
//	2008-06-21	Andreas Mayer
//	- fixed -imageByScalingToSize:preserveAspectRatio: to correctly determine
//	  target size when preserving aspect ratio


#import "NSImage_AMAdditions.h"


@implementation NSImage (AMAdditions)

// If a bitmap image, fix the size of the bitmap so that it is
// equal to the exact pixel dimensions.
// FROM: Dan Wood
// DATE: 2001-10-19 19:00

- (NSImage *)normalizeSize
{
	NSBitmapImageRep *theBitmap = nil;
	NSSize newSize;
	NSArray *reps = [self representations];
	int i;

	for (i = 0 ; i < [reps count] ; i++ )
	{
		NSImageRep *theRep = [reps objectAtIndex:i];
		if ([theRep isKindOfClass:[NSBitmapImageRep class]])
		{
			theBitmap = (NSBitmapImageRep *)theRep;
			break;
		}
	}
	if (nil != theBitmap)
	{
		newSize.width = [theBitmap pixelsWide];
		newSize.height = [theBitmap pixelsHigh];
		[theBitmap setSize:newSize];
		[self setSize:newSize];
	}
	return self;
}

- (NSImage *)darkenedImageWithColor:(NSColor *)tint
{
	NSImage *result = nil;
	NSSize size = [self size];
	
	// get RGB components from tint color
	NSColor *rgbTint = [tint colorUsingColorSpaceName:NSCalibratedRGBColorSpace];
	// get bitmap data
	NSImageRep *imageRep = [[self representations] objectAtIndex:0];
	if ([imageRep isKindOfClass:NSClassFromString(@"NSBitmapImageRep")]) {
		unsigned char *sourceData = [(NSBitmapImageRep *)imageRep bitmapData];
		NSInteger bytesPerRow = [(NSBitmapImageRep *)imageRep bytesPerRow];
		NSInteger bytesPerPixel = [(NSBitmapImageRep *)imageRep bitsPerPixel]/8;
		BOOL hasAlpha = [(NSBitmapImageRep *)imageRep hasAlpha];

		NSBitmapImageRep *bitmapImageRep;
		if (hasAlpha) {
			bitmapImageRep = [[NSBitmapImageRep alloc] initWithBitmapDataPlanes:nil pixelsWide:size.width pixelsHigh:size.height bitsPerSample:8 samplesPerPixel:4 hasAlpha:YES isPlanar:NO colorSpaceName:NSCalibratedRGBColorSpace bytesPerRow:0 bitsPerPixel:0];
		} else {
			bitmapImageRep = [[NSBitmapImageRep alloc] initWithBitmapDataPlanes:nil pixelsWide:size.width pixelsHigh:size.height bitsPerSample:8 samplesPerPixel:3 hasAlpha:NO isPlanar:NO colorSpaceName:NSCalibratedRGBColorSpace bytesPerRow:0 bitsPerPixel:0];
		}
		result = [[[NSImage alloc] initWithSize:size] autorelease];
		[result addRepresentation:bitmapImageRep];
		
		unsigned char *destData = [bitmapImageRep bitmapData];
		NSInteger x, y, pos;
		int value = 0;
		int redTint = (256 - [rgbTint redComponent]*255);
		int greenTint = (256 - [rgbTint greenComponent]*255);
		int blueTint = (256 - [rgbTint blueComponent]*255);
		// process pixels
		for (x = 0; x < size.width; x++) {
			for (y = 0; y < size.height; y++) {
				pos = (y * bytesPerRow) + (x * bytesPerPixel);
				value = sourceData[pos] - redTint;
				destData[pos] = ((value > 0) ? value : 0);
				value = sourceData[pos+1] - greenTint;
				destData[pos+1] = ((value > 0) ? value : 0);
				value = sourceData[pos+2] - blueTint;
				destData[pos+2] = ((value > 0) ? value : 0);
				if (hasAlpha) {
					// copy alpha from source
					destData[pos+3] = sourceData[pos+3];
				}
			}
		}
		[bitmapImageRep release];
	} else {
		NSLog(@"not a bitmap image rep");
	}
	return result;
}

- (NSImage *)tintedImageWithColor:(NSColor *)tint
{
	NSImage *result = [[[NSImage alloc] initWithSize:[self size]] autorelease];
	[result lockFocus];
	[tint set];
	NSRect rect = NSZeroRect;
	rect.size = [self size];
	NSRectFill(rect);
	[self drawAtPoint:NSZeroPoint fromRect:NSZeroRect operation:NSCompositeDestinationIn fraction:1.0];
	[result unlockFocus];
	return result;
}


- (NSImage *)imageByScalingToSize:(NSSize)newSize preserveAspectRatio:(BOOL)preserveAspectRatio
{
	NSImage *result = nil;
	NSSize oldSize = [self size];
	
	if (NSEqualSizes(oldSize, newSize)) {
		result = [[self retain] autorelease];
	} else {
		float ratioOld = oldSize.width/oldSize.height;
		float ratioNew = newSize.width/newSize.height;
		if (preserveAspectRatio && (ratioOld != ratioNew)) {
			if (ratioOld > ratioNew) {
				newSize.height = oldSize.height*(newSize.width/oldSize.width);
			} else {
				newSize.width = oldSize.width*(newSize.height/oldSize.height);
			}
		}
//		NSLog(@"old size: %f * %f, new size: %f * %f", oldSize.width, oldSize.height, newSize.width, newSize.height);
		
		result = [[[NSImage alloc] initWithSize:newSize] autorelease];
		NSRect sourceRect = NSMakeRect(0, 0, oldSize.width, oldSize.height);
		NSRect destRect = NSMakeRect(0, 0, newSize.width, newSize.height);
		
		[result lockFocus];
		[[NSGraphicsContext currentContext] setImageInterpolation:NSImageInterpolationHigh];
		[self drawInRect:destRect fromRect:sourceRect operation:NSCompositeCopy fraction:1.0];
		[result unlockFocus];
	}
	return result;
}


@end
